
#include "shared.h"

int comms_port = 0x320;

/*
    Initialize Comms Link library
*/
void comms_init(void)
{
    atexit(comms_shutdown);
}


/*
    Set Comms Link port
*/
void comms_set_port(int port)
{    
    comms_port = port;
}

/*
    Clear Comms Link data out latch. Having too many bits set in the latch
    will provide enough power to the Saturn to prevent it from booting up.
*/
void comms_shutdown(void)
{
    if(comms_port)
        outp(comms_port, 0x00);
}


uint8 comms_exchange_byte(uint8 value)
{
    outp(comms_port, value);
    while (inp(comms_port + 2) & 1);
    return inp(comms_port);
}


void comms_send_word(uint16 data)
{
    comms_exchange_byte(data >> 8);
    comms_exchange_byte(data & 0xFF);
}


void comms_send_long(uint32 data)
{
    comms_exchange_byte(data >> 24);
    comms_exchange_byte((data >> 16) & 0xFF);
    comms_exchange_byte((data >> 8) & 0xFF);
    comms_exchange_byte(data & 0xFF);
}


uint32 comms_get_long(void)
{
    uint32 data = comms_exchange_byte(0) << 24;
    data |= comms_exchange_byte(0) << 16;
    data |= comms_exchange_byte(0) << 8;
    data |= comms_exchange_byte(0);
    return data;
}

/*--------------------------------------------------------------------------*/
/* Saturn interface functions                                               */
/*--------------------------------------------------------------------------*/

/*
    Synchronize with PAR 
*/
void par_sync(void)
{
    while (1)
    {
        __dpmi_yield();
        if (comms_exchange_byte('I') != 'D')
            continue;
        if (comms_exchange_byte('N') == 'O')
            break;
    }
}


/*
    Returns
        1= No error
        0= Invalid Comms Link port specified
       -1= Checksum mismatch
       -2= Couldn't terminate download request successfully
*/
int par_dump_memory(uint8 *buffer, uint32 address, uint32 length)
{
    int i, error = 1;
    uint8 checksum[2];

    /* Synchronize with PAR */
    par_sync();

    /* Function $01 - Download memory */
    comms_exchange_byte(1);

    /* Read and discard contents of master SH-2 register R9 */
    comms_get_long();

    /* Send source address and length to read */
    comms_send_long(address);
    comms_send_long(length);

    /* Read data and generate checksum */
    for (i = 0; i < length; i++)
    {
        buffer[i] = comms_exchange_byte(0);
        checksum[0] += buffer[i];
    }

    /* See if the checksums match */
    checksum[1] = comms_exchange_byte(0);
    if(checksum[0] != checksum[1])
        error = -1;

    /* Abort download - dummy address with zero length */
    comms_send_long(0);
    comms_send_long(0);

    /* See if the PAR acknowledge request to stop download */
    if(comms_exchange_byte(0) != 'O') error = -2;
    if(comms_exchange_byte(0) != 'K') error = -2;

    return error;
}

/*
    Returns
        1= No error
        0= Invalid Comms Link port specified
*/
int par_load_data(uint8 *buffer, uint32 address, uint32 length, uint8 exec_flag)
{
    int i;

    /* Synchronize with PAR */
    par_sync();

    /* Function $09 - Upload data */
    comms_exchange_byte(9);

    /* Send destination address and data length */
    comms_send_long(address);
    comms_send_long(length);

    /* Send load-only or load-execute flag */
    comms_exchange_byte(exec_flag);

    /* Send data */
    for (i = 0; i < length; i++)
        comms_exchange_byte(buffer[i]);

    return 1;
}

