/*
    Saturn PAR utility
    by Charles MacDonald
    WWW: http://cgfm2.emuviews.com

    Based on code by Bart Trzynadlowski and James Forshaw.

    Copyright (C) 2002  Charles MacDonald

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "shared.h"

int verbose;

option_t program_options[] =
{
    {"-v",  2, &opt_verbose   },
    {"-p",  2, &opt_set_port  },
    {"-l",  2, &opt_load_data },
    {"-x",  2, &opt_exec_data },
    {"-d",  3, &opt_dump_mem  },
    {"-db", 1, &opt_dump_bios },
    {"-df", 1, &opt_dump_firm },
    {"-bs", 1, &opt_save_bram },
    {"-bl", 1, &opt_load_bram },
    {NULL, 0, NULL}
};

int main (int argc, char *argv[])
{
    char *p;

    verbose = 0;
    comms_init();

    /* Check for AR_PORT environment variable */
    p = getenv("AR_PORT");
    if(p)
        comms_set_port(strtoul(p, NULL, 16));

    parse_args(argc, argv);

    return 0;
}

void print_help(char *argv[])
{
    printf("Saturn PAR utility by Charles MacDonald (http://cgfm2.emuviews.com)\n");
    printf("Based on code by Bart Trzynadlowski and James Forshaw.\n\n");
    printf("usage: %s [-options] [-switches]\n\n", argv[0]);
    printf("Available options are:\n");
    printf(" -p  <port>               Specify Comms Link port (default: 0x320)\n");
    printf(" -v                       Be verbose\n");
    printf("Available switches are:\n");
    printf(" -l  <file> <addr>        Load file to address\n");
    printf(" -x  <file> <addr>        Load file and JSR to load address\n");
    printf(" -d  <file> <addr> <size> Dump data to file\n");
    printf("Some shortcuts for the above switches:\n");
    printf(" -db <file>               Dump BIOS (assume 512K)\n");
    printf(" -df <file>               Dump cheat cartridge firmware (assume 256K)\n");
    printf(" -bs <file>               Save battery backed RAM to file\n");
    printf(" -bl <file>               Load battery backed RAM from file (must be 32K)\n");
    printf("\n");
    printf("Uses AR_PORT environment variable if available. (-p overrides)\n");
    printf("Ensure hexadecimal values are preceeded with `0x'.\n");
}

/*--------------------------------------------------------------------------*/

int opt_set_port(int index, char *argv[])
{
    int port = strtoul(argv[index], NULL, 0);

    if(port < 0x320 || port > 0x330)
        port = 0x320;

    if(verbose)
        printf("Using port %04X\n", port);

    comms_set_port(port);
    return 0;
}

int opt_verbose(int index, char *argv[])
{
    verbose = 1;
    return 0;
}

int opt_load_data(int index, char *argv[])
{
    FILE *fd;
    uint8 *buffer;
    uint32 address, length;

    address = strtoul(argv[index+1], NULL, 0);

    fd = fopen(argv[index], "rb");
    if(!fd) return 0;
    fseek(fd, 0, SEEK_END);
    length = ftell(fd);
    fseek(fd, 0, SEEK_SET);

    buffer = malloc(length);
    if(!buffer) {
        fclose(fd);
        return 0;
    }
    fread(buffer, length, 1, fd);
    fclose(fd);

    if(verbose)
        printf("Loading file `%s' to address %08X\n", argv[index], address);
    par_load_data(buffer, address, length, FLAG_LOAD);

    free(buffer);
    return 1;
}

int opt_exec_data(int index, char *argv[])
{
    FILE *fd;
    uint8 *buffer;
    uint32 address, length;

    address = strtoul(argv[index+1], NULL, 0);

    fd = fopen(argv[index], "rb");
    if(!fd) return 0;

    fseek(fd, 0, SEEK_END);
    length = ftell(fd);
    fseek(fd, 0, SEEK_SET);

    buffer = malloc(length);
    if(!buffer) {
        fclose(fd);
        return 0;
    }
    fread(buffer, length, 1, fd);
    fclose(fd);

    if(verbose)
        printf("Loading file `%s' to address %08X\n", argv[index], address);
    par_load_data(buffer, address, length, FLAG_EXEC);

    free(buffer);
    return 1;
}

int opt_dump_mem(int index, char *argv[])
{
    FILE *fd;
    uint8 *buffer;
    uint32 address, length;

    address = strtoul(argv[index+1], NULL, 0);
    length  = strtoul(argv[index+2], NULL, 0);

    fd = fopen(argv[index], "wb");
    if(!fd) return 0;

    buffer = malloc(length);
    if(!buffer) {
        fclose(fd);
        return 0;
    }

    if(verbose)
        printf("Reading data from %08X-%08X to `%s'\n", address, address+(length-1), argv[index]);
    par_dump_memory(buffer, address, length);

    fwrite(buffer, length, 1, fd);
    fclose(fd);
    free(buffer);

    return 1;
}

int opt_dump_bios(int index, char *argv[])
{
    FILE *fd;
    uint8 *buffer;

    fd = fopen(argv[index], "wb");
    if(!fd) return 0;

    buffer = malloc(0x80000);
    if(!buffer) {
        fclose(fd);
        return 0;
    }

    printf("Dumping BIOS to `%s'\n", argv[index]);
    par_dump_memory(buffer, 0x20000000, 0x80000);

    fwrite(buffer, 0x80000, 1, fd);
    fclose(fd);
    free(buffer);

    return 1;
}

int opt_dump_firm(int index, char *argv[])
{
    FILE *fd;
    uint8 *buffer;

    fd = fopen(argv[index], "wb");
    if(!fd) return 0;

    buffer = malloc(0x40000);
    if(!buffer) {
        fclose(fd);
        return 0;
    }

    printf("Dumping cheat cartridge firmware to `%s'\n", argv[index]);
    par_dump_memory(buffer, 0x22000000, 0x40000);

    fwrite(buffer, 0x40000, 1, fd);
    fclose(fd);
    free(buffer);

    return 1;
}

int opt_save_bram(int index, char *argv[])
{
    int i;
    FILE *fd;
    uint8 buffer[0x10000];

    fd = fopen(argv[index], "wb");
    if(!fd) return 0;

    printf("Saving battery backed RAM to `%s'\n", argv[index]);
    par_dump_memory(buffer, 0x20180000, 0x10000);

    for(i = 0; i < 0x8000; i++)
        buffer[i] = buffer[i*2+1];

    fwrite(buffer, 0x8000, 1, fd);
    fclose(fd);


    return 1;
}

int opt_load_bram(int index, char *argv[])
{
    int i;
    FILE *fd;
    uint8 bram[0x8000];
    uint8 buffer[0x10000];

    fd = fopen(argv[index], "rb");
    if(!fd) return 0;
    fread(bram, 0x8000, 1, fd);
    fclose(fd);

    for(i = 0; i < 0x8000; i++)
    {
        buffer[i*2+0] = 0xFF;
        buffer[i*2+1] = bram[i];
    }

    printf("Loading battery backed RAM from `%s'\n", argv[index]);
    par_load_data(buffer, 0x20180000, 0x10000, 0x00);

    return 1;
}

/*--------------------------------------------------------------------------*/

void parse_args(int argc, char *argv[])
{
    int i = 1, j;

    /* No arguments given */
    if(argc < 1)
        return;

    /* Print help if invoked with no arguments or '--help' option used */
    if(argc == 1 || stricmp(argv[1], "--help") == 0) {
        print_help(argv);
        return;
    }

    for(i = 1; i < argc; i++)
    {
        for(j = 0; program_options[j].func != NULL; j++)
        {
            if(stricmp(argv[i], program_options[j].text) == 0)
            {
                    program_options[j].func(i+1, argv);
            }
        }
    }
}
